package de.xoev.xta.test.app.config;

import java.util.Collection;
import java.util.List;

import org.apache.cxf.bus.spring.SpringBus;
import org.apache.cxf.ext.logging.LoggingInInterceptor;
import org.apache.cxf.ext.logging.LoggingOutInterceptor;
import org.apache.cxf.interceptor.Interceptor;
import org.apache.cxf.jaxws.EndpointImpl;
import org.apache.cxf.message.Message;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import de.xoev.xta.test.app.service.interceptor.ServerProtocolFaultOutInterceptor;
import de.xoev.xta.test.app.service.interceptor.ServerProtocolInInterceptor;
import de.xoev.xta.test.app.service.interceptor.ServerProtocolOutInterceptor;
import genv3.de.xoev.transport.xta.x211.ManagementPortType;
import genv3.de.xoev.transport.xta.x211.MsgBoxPortType;
import genv3.de.xoev.transport.xta.x211.SendPortType;
import jakarta.xml.ws.Endpoint;
import jakarta.xml.ws.soap.SOAPBinding;
import lombok.extern.log4j.Log4j2;

@Configuration
@Log4j2
public class XTAServer {

    @Autowired
    private SpringBus bus;

    @Autowired
    private ServerProtocolInInterceptor serverProtocolInInterceptor;

    @Autowired
    private ServerProtocolFaultOutInterceptor serverProtocolFaultOutInterceptor;

    @Autowired
    private ServerProtocolOutInterceptor serverProtocolOutInterceptor;

    Collection<Interceptor<? extends Message>> getDefaultInInterceptors() {
        return List.of(serverProtocolInInterceptor, new LoggingInInterceptor());
    }

    Collection<Interceptor<? extends Message>> getDefaultFaultOutInterceptors() {
        return List.of(serverProtocolFaultOutInterceptor, new LoggingOutInterceptor());
    }

    Collection<Interceptor<? extends Message>> getDefaultOutInterceptors() {
        return List.of(serverProtocolOutInterceptor, new LoggingOutInterceptor());
    }

    @Bean
    Endpoint managementEndpoint(final ManagementPortType managementPortType) {
        EndpointImpl endpoint = new EndpointImpl(bus, managementPortType);
        endpoint.publish("/XTAService/ManagementPort");
        configureEndpoint(endpoint);
        return endpoint;
    }

    @Bean
    Endpoint msgBoxEndpoint(final MsgBoxPortType msgBoxPortType) {
        EndpointImpl endpoint = new EndpointImpl(bus, msgBoxPortType);
        endpoint.publish("/XTAService/MsgBoxPort");
        configureEndpoint(endpoint);
        return endpoint;
    }

    @Bean
    Endpoint sendXtaEndpoint(final SendPortType sendPortType) {
        EndpointImpl endpoint = new EndpointImpl(bus, sendPortType);
        endpoint.publish("/XTAService/SendXtaPort");
        configureEndpoint(endpoint);
        return endpoint;
    }

    private void configureEndpoint(final EndpointImpl endpoint) {
        log.debug("configuring endpoint {}", endpoint.getBeanName());
        Collection<Interceptor<? extends Message>> defaultFaultOutInterceptor = getDefaultFaultOutInterceptors();
        Collection<Interceptor<? extends Message>> defaultOutInterceptor = getDefaultOutInterceptors();
        Collection<Interceptor<? extends Message>> defaultInInterceptor = getDefaultInInterceptors();
        endpoint.getInInterceptors().addAll(defaultInInterceptor);
        endpoint.getOutFaultInterceptors().addAll(defaultFaultOutInterceptor);
        endpoint.getInFaultInterceptors().addAll(defaultFaultOutInterceptor);
        endpoint.getOutInterceptors().addAll(defaultOutInterceptor);
        SOAPBinding binding = (SOAPBinding) endpoint.getBinding();
        binding.setMTOMEnabled(true);
    }
}
