/*
* @formatter:off
*
* Copyright 2021-2022  Koordinierungsstelle für IT-Standards (KoSIT)
*
* Licensed under the European Public License, Version 1.2 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     https://opensource.org/licenses/EUPL-1.2
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
* @formatter:on
*/
package de.xoev.xta.test.app.model;

import javax.xml.namespace.QName;

import lombok.experimental.UtilityClass;

@UtilityClass
public class XtaConstants {

    public static final String W3_ORG_2005_08_ADDRESSING = "http://www.w3.org/2005/08/addressing";

    public static final String XTA_211 = "http://xoev.de/transport/xta/211";
    public static final String XTA_ACTION_BASE = "http://www.xta.de/XTA/";

    public static final String OSCI_2008_05_TRANSPORT = "http://www.osci.eu/ws/2008/05/transport";
    public static final String OSCI_2014_10_TRANSPORT = "http://www.osci.eu/ws/2014/10/transport";

    public static final String OSCI_2008_05_ACTION_BASE = OSCI_2008_05_TRANSPORT + "/urn/messageTypes/";
    public static final String OSCI_2014_10_ACTION_BASE = OSCI_2014_10_TRANSPORT + "/urn/messageTypes/";

    /*
     * Xml QNames in alphabetic order
     */

    public static final QName ACTION = new QName(W3_ORG_2005_08_ADDRESSING, "Action");
    public static final QName MSG_BOX_RESPONSE = new QName(OSCI_2008_05_TRANSPORT, "MsgBoxResponse");
    public static final QName MESSAGE_META_DATA = new QName(OSCI_2014_10_TRANSPORT, "MessageMetaData");
    public static final QName TO = new QName(W3_ORG_2005_08_ADDRESSING, "To");

    /*
     * Actions in alphabetic order
     */

    public static final String CANCEL_MESSAGE_ACTION = XTA_ACTION_BASE + "CancelMessage";
    public static final String CHECK_ACCOUNT_ACTIVE_ACTION = XTA_ACTION_BASE + "CheckAccountActive";
    public static final String CLOSE_ACTION = OSCI_2008_05_ACTION_BASE + "MsgBoxCloseRequest";
    public static final String CREATE_MESSAGE_ID_ACTION = XTA_ACTION_BASE + "CreateMessageID";

    public static final String GET_MESSAGE_ACTION = OSCI_2008_05_ACTION_BASE + "MsgBoxFetchRequest";
    public static final String GET_NEXT_MESSAGE_ACTION = OSCI_2008_05_ACTION_BASE + "MsgBoxGetNextMsgRequest";
    public static final String GET_NEXT_STATUS_LIST_ACTION = OSCI_2008_05_ACTION_BASE + "MsgBoxGetNextListRequest";
    public static final String GET_STATUS_LIST_ACTION = OSCI_2008_05_ACTION_BASE + "MsgBoxStatusListRequest";
    public static final String GET_TRANSPORT_REPORT_ACTION = XTA_ACTION_BASE + "GetTransportReport";

    public static final String LOOKUP_SERVICE_ACTION = XTA_ACTION_BASE + "IsServiceAvailable";

    public static final String SEND_MESSAGE_ACTION = XTA_ACTION_BASE + "SendMessage";
    public static final String SEND_MESSAGE_SYNC_ACTION = XTA_ACTION_BASE + "SendMessageSync";

    public enum ConnectionPortType {
        SEND, MSGBOX, MANAGEMENT
    }
}
