
package de.xoev.xta.test.app.server;

import java.io.IOException;
import java.math.BigInteger;
import java.util.List;
import java.util.Optional;

import org.apache.cxf.annotations.SchemaValidation;
import org.apache.cxf.ws.addressing.AttributedURIType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Component;

import de.xoev.xta.test.app.service.ParameterValidatorService;
import de.xoev.xta.test.app.service.ProtocolService;
import de.xoev.xta.test.app.service.XtaParameter;
import de.xoev.xta.test.app.util.UtilComponent;
import de.xoev.xta.test.app.util.XtaIdGenerator;
import genv3.de.xoev.transport.xta.x211.AdditionalReportListType;
import genv3.de.xoev.transport.xta.x211.AdditionalReportListType.Report;
import genv3.de.xoev.transport.xta.x211.CancelDeniedException;
import genv3.de.xoev.transport.xta.x211.CodeReportType;
import genv3.de.xoev.transport.xta.x211.InvalidMessageIDException;
import genv3.de.xoev.transport.xta.x211.IsServiceAvailableValueType;
import genv3.de.xoev.transport.xta.x211.LookupServiceRequest;
import genv3.de.xoev.transport.xta.x211.LookupServiceRequest.LookupServiceRequestList;
import genv3.de.xoev.transport.xta.x211.LookupServiceResponse;
import genv3.de.xoev.transport.xta.x211.LookupServiceResponse.LookupServiceResultList;
import genv3.de.xoev.transport.xta.x211.LookupServiceResultType;
import genv3.de.xoev.transport.xta.x211.ManagementPortType;
import genv3.de.xoev.transport.xta.x211.ParameterIsNotValidException;
import genv3.de.xoev.transport.xta.x211.PermissionDeniedException;
import genv3.de.xoev.transport.xta.x211.TransportReport;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import genv3.eu.osci.ws.x2014.x10.transport.PartyType;
import jakarta.activation.DataHandler;
import jakarta.annotation.Generated;
import jakarta.jws.WebService;
import lombok.extern.log4j.Log4j2;

@WebService(serviceName = "XTAService", portName = "ManagementPort", targetNamespace = "http://xoev.de/transport/xta/211", wsdlLocation = "classpath:wsdl/XTA.wsdl", endpointInterface = "genv3.de.xoev.transport.xta.x211.ManagementPortType")
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
@Component
@Log4j2
public class ManagementPortImpl implements ManagementPortType {

    @Autowired
    private ParameterValidatorService validator;

    @Autowired
    private ProtocolService protocolService;

    @Autowired
    private UtilComponent utilComponent;

    /*
     * (non-Javadoc)
     * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#createMessageId(genv3.eu.osci.ws.x2014.x10.transport.
     * PartyType authorIdentifier)*
     */
    @Override
    public AttributedURIType createMessageId(
            final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
            throws XTAWSTechnicalProblemException, PermissionDeniedException {
        log.info("Executing operation createMessageId");
        AttributedURIType messageId = new AttributedURIType();
        messageId.setValue(XtaIdGenerator.generateMessageId().toString());
        protocolService.addParameter(XtaParameter.MESSAGE_ID, messageId.getValue());
        return messageId;
    }

    /*
     * (non-Javadoc)
     * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#checkAccountActive(genv3.eu.osci.ws.x2014.x10.transport.
     * PartyType authorIdentifier)*
     */
    @Override
    public void checkAccountActive(final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
            throws XTAWSTechnicalProblemException, PermissionDeniedException {
        log.info("Executing operation checkAccountActive");
        log.info("checkAccountActive called, author: {}", authorIdentifier);
    }

    /*
     * (non-Javadoc)
     * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#lookupService(genv3.de.xoev.transport.xta.x211.
     * LookupServiceRequest lookupServiceRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
     */
    @Override
    @Generated(value = "org.apache.cxf.tools.wsdlto.WSDLToJava", date = "2023-03-01T15:42:39.751+01:00")
    public genv3.de.xoev.transport.xta.x211.LookupServiceResponse lookupService(
            final LookupServiceRequest lookupServiceRequest,
            final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
            throws XTAWSTechnicalProblemException, PermissionDeniedException, ParameterIsNotValidException {
        log.info("Executing operation lookupService");

        LookupServiceResponse lookupResponse = new LookupServiceResponse();

        LookupServiceResultList lookupServiceResultList = new LookupServiceResultList();
        lookupResponse.setLookupServiceResultList(lookupServiceResultList);
        List<LookupServiceResultType> resultlist = lookupServiceResultList.getLookupServiceResult();

        for (LookupServiceRequestList lookupServiceRequestList : lookupServiceRequest.getLookupServiceRequestList()) {

            LookupServiceResultType lookupServiceResult = new LookupServiceResultType();
            resultlist.add(lookupServiceResult);
            IsServiceAvailableValueType serviceAvailable = new IsServiceAvailableValueType();
            lookupServiceResult.setIsServiceAvailableValue(serviceAvailable);
            serviceAvailable.setServiceIsAvailable(true);

            PartyType requestReader = lookupServiceRequestList.getLookupService().getReader();
            PartyType reader = new PartyType();
            lookupServiceResult.setReader(reader);
            reader.setIdentifier(requestReader.getIdentifier());
            reader.getSecurityToken().addAll(requestReader.getSecurityToken());

            lookupServiceResult.setServiceType(lookupServiceRequestList.getLookupService().getServiceType());
        }
        return lookupResponse;
    }

    /*
     * (non-Javadoc)
     * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#cancelMessage(genv3.org.w3.x2005.x08.addressing.
     * AttributedURIType messageID, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
     */
    @Override
    @Generated(value = "org.apache.cxf.tools.wsdlto.WSDLToJava", date = "2023-03-01T15:42:39.751+01:00")
    public void cancelMessage(final AttributedURIType messageID,
            final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
            throws XTAWSTechnicalProblemException, CancelDeniedException, PermissionDeniedException,
            ParameterIsNotValidException {
        log.info("Executing operation cancelMessage");
        log.info("cancelMessage called, author: {}", authorIdentifier);

    }

    /*
     * (non-Javadoc)
     * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#getTransportReport(genv3.org.w3.x2005.x08.addressing.
     * AttributedURIType messageID, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
     */
    @Override
    @Generated(value = "org.apache.cxf.tools.wsdlto.WSDLToJava", date = "2023-03-01T15:42:39.751+01:00")
    public genv3.de.xoev.transport.xta.x211.TransportReport getTransportReport(
            final AttributedURIType messageID,
            final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
            throws XTAWSTechnicalProblemException, InvalidMessageIDException, PermissionDeniedException {
        log.info("Executing operation getTransportReport");
        Optional<String> expectedMessageId = protocolService.getParameter(XtaParameter.MESSAGE_ID, String.class);
        if (validator.isNotBlank(messageID.getValue(), XtaParameter.MESSAGE_ID) && expectedMessageId.isPresent()) {
            validator.isEquals(expectedMessageId.get(), messageID.getValue(), XtaParameter.MESSAGE_ID);
        }
        TransportReport transportReport = new TransportReport();

        transportReport.setReportTime(UtilComponent.createGC(0));
        transportReport.setXTAServerIdentity("xta-tester");
        MessageMetaData messageMetaData = new MessageMetaData();
        utilComponent.createMessageMetaData(messageMetaData, messageID.getValue());
        transportReport.setMessageMetaData(messageMetaData);

        AdditionalReportListType additionalReportListType = new AdditionalReportListType();
        transportReport.setAdditionalReports(additionalReportListType);

        Report report = new Report();
        CodeReportType codeReportType = new CodeReportType();
        codeReportType.setCode("code");
        codeReportType.setListURI("uri:otherListUri");
        codeReportType.setListVersionID("version");

        report.setKey(codeReportType);
        try {
            report.setData(new DataHandler(new ClassPathResource("static/output.png").getURL()));
        } catch (IOException e) {
            log.debug("Could not load image {}: {}", e.getMessage(), e);
        }
        additionalReportListType.getReport().add(report);

        transportReport.getMessageStatus().setStatus(BigInteger.ONE);

        return transportReport;
    }

}
