/*
 * Created 2023-03-15
 */
package de.xoev.xta.test.app.service;

import java.util.Collection;
import java.util.List;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.ext.logging.LoggingInInterceptor;
import org.apache.cxf.ext.logging.LoggingOutInterceptor;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.Interceptor;
import org.apache.cxf.message.Message;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.xoev.xta.test.app.exception.XtaTesterException;
import de.xoev.xta.test.app.service.interceptor.ClientProtocolInInterceptor;
import de.xoev.xta.test.app.service.interceptor.ClientProtocolOutInterceptor;
import de.xoev.xta.test.app.service.interceptor.ServerProtocolFaultOutInterceptor;
import genv3.de.xoev.transport.xta.x211.ManagementPortType;
import genv3.de.xoev.transport.xta.x211.MsgBoxPortType;
import genv3.de.xoev.transport.xta.x211.SendPortType;
import genv3.de.xoev.transport.xta.x211.XTAService;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
public class ClientConfigurationContainer {

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private ClientProtocolInInterceptor clientProtocolInInterceptor;

    @Autowired
    private ServerProtocolFaultOutInterceptor serverProtocolFaultOutInterceptor;

    @Autowired
    private ClientProtocolOutInterceptor clientProtocolOutInterceptor;

    private LoggingInInterceptor createLoggingInInterceptor() {
        var i = new LoggingInInterceptor();
        i.setPrettyLogging(true);
        i.setLogBinary(true);
        i.setLogMultipart(true);
        return i;
    }

    private LoggingOutInterceptor createLoggingOutInterceptor() {
        var i = new LoggingOutInterceptor();
        i.setPrettyLogging(true);
        i.setLogBinary(true);
        i.setLogMultipart(true);
        return i;
    }

    Collection<Interceptor<? extends Message>> getDefaultInInterceptors() {
        return List.of(clientProtocolInInterceptor, createLoggingInInterceptor());
    }

    Collection<Interceptor<? extends Message>> getDefaultFaultOutInterceptors() {
        return List.of(serverProtocolFaultOutInterceptor, createLoggingOutInterceptor());
    }

    Collection<Interceptor<? extends Message>> getDefaultOutInterceptors() {
        return List.of(clientProtocolOutInterceptor, createLoggingOutInterceptor());
    }

    Collection<Interceptor<? extends Message>> getDefaultFaultInInterceptors() {
        // doesn't matter that it's named serverProtocolFault_OUT_Interceptor
        return List.of(serverProtocolFaultOutInterceptor, createLoggingInInterceptor());
    }

    private XTAService getStub() {
        return new XTAService(XTAService.WSDL_LOCATION);
    }

    public ManagementPortType getManagementPortClient() throws XtaTesterException {
        ManagementPortType managementPortType = getStub().getManagementPort();
        Client client = ClientProxy.getClient(managementPortType);
        setClientProperties(client, configurationService.getClientProperties().getServerUrl().getManagementPort());
        return managementPortType;
    }

    public MsgBoxPortType getMsgBoxPortClient() throws XtaTesterException {
        MsgBoxPortType msgBoxPortType = getStub().getMsgBoxPort();
        Client client = ClientProxy.getClient(msgBoxPortType);
        setClientProperties(client, configurationService.getClientProperties().getServerUrl().getMsgBoxPort());
        return msgBoxPortType;
    }

    public SendPortType getSendPortClient() throws XtaTesterException {
        SendPortType sendPortType = getStub().getSendXtaPort();
        Client client = ClientProxy.getClient(sendPortType);
        setClientProperties(client, configurationService.getClientProperties().getServerUrl().getSendPort());
        return sendPortType;
    }

    private void setClientProperties(final Client client, final String url) throws XtaTesterException {
        // set remote endpoint
        client.getRequestContext().put(Message.ENDPOINT_ADDRESS, url);
        client.getInInterceptors().addAll(getDefaultInInterceptors());
        client.getOutFaultInterceptors().addAll(getDefaultFaultOutInterceptors());
        client.getInFaultInterceptors().addAll(getDefaultFaultInInterceptors());
        client.getOutInterceptors().addAll(getDefaultOutInterceptors());
    }

}
