/*
 * Created 2023-03-24
 */
package de.xoev.xta.test.app.service;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.xoev.xta.test.app.config.MapStructConfiguration;
import de.xoev.xta.test.app.config.XtaConfig;
import de.xoev.xta.test.app.config.XtaConfigQualifierType;
import de.xoev.xta.test.app.model.XtaConfigDto;
import de.xoev.xta.test.app.model.XtaIdentifierConfig;
import genv3.eu.osci.ws.x2014.x10.transport.KeyCodeType;
import genv3.eu.osci.ws.x2014.x10.transport.PartyType;
import genv3.eu.osci.ws.x2014.x10.transport.QualifierType;

@Mapper(config = MapStructConfiguration.class)
public interface XtaConfigDtoConverter {

    XtaConfigDto fromModel(final XtaConfig model);

    @Mapping(target = "securityToken", ignore = true)
    @Mapping(target = "identifier.type", source = "identifierScheme")
    @Mapping(target = "identifier.category", source = "organizationCategory")
    @Mapping(target = "identifier.name", source = "name")
    @Mapping(target = "identifier.value", source = "value")
    PartyType fromModel(final XtaIdentifierConfig xtaIdentifierConfig);

    default QualifierType qualifierFromConfig(final XtaConfigQualifierType cfg) {
        final var messageType = new QualifierType.MessageType();
        messageType.setPayloadSchema(cfg.getMessageTypePayloadSchema());
        messageType.setName(cfg.getMessageTypeName());
        messageType.setCode(cfg.getMessageTypeCode());
        messageType.setListURI(cfg.getMessageTypeListUri());
        messageType.setListVersionID(cfg.getMessageTypeListVersionId());

        final var business = new QualifierType.BusinessScenario();

        if (cfg.isBusinessScenarioUseUndefined()) {
            business.setUndefined(cfg.getBusinessScenarioUndefined());
            business.setDefined(null);
        } else {
            final var businessDefined = new KeyCodeType();
            businessDefined.setName(cfg.getBusinessScenarioName());
            businessDefined.setCode(cfg.getBusinessScenarioCode());
            businessDefined.setListURI(cfg.getBusinessScenarioListUri());
            businessDefined.setListVersionID(cfg.getBusinessScenarioListVersionId());
            business.setDefined(businessDefined);
            business.setUndefined(null);
        }

        final var qualifierType = new QualifierType();
        qualifierType.setSubject(cfg.getQualifierSubject());
        qualifierType.setService(cfg.getQualifierService());
        qualifierType.setBusinessScenario(business);
        qualifierType.setMessageType(messageType);

        return qualifierType;
    }
}
